﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Statki.Model;

namespace Statki.Tests
{
    [TestClass]
    public class AITests
    {
        [TestMethod]
        public void ShouldCalculateCorrectDirection()
        {
            var testData = new Tuple<Coordinate[], Direction>[]
            {
                new Tuple<Coordinate[], Direction>(
                    new Coordinate[]
                    {
                        new Coordinate(2, 1),
                        new Coordinate(4, 1)
                    }, Direction.Horizontal),

                new Tuple<Coordinate[], Direction>(
                    new Coordinate[]
                    {
                        new Coordinate(3, 1),
                        new Coordinate(3, 2)
                    }, Direction.Vertical),

                new Tuple<Coordinate[], Direction>(
                    new Coordinate[]
                    {
                        new Coordinate(7, 5),
                        new Coordinate(10, 5)
                    }, Direction.Horizontal)
            };

            foreach (var testEntry in testData)
            {
                Direction result = AI.CalculateDirection(testEntry.Item1);
                Assert.AreEqual(testEntry.Item2, result);
            }
        }
        [TestMethod]
        //testuje dla jednego trafionego pola
        public void PotentialFields1()
        {
            var testData = Fixtures.GenerateFieldMatrix(5);
            Ship ship = new Ship(new System.Collections.Generic.List<Field>() { testData[1, 1], testData[2, 1], testData[3, 1] });
            testData[1, 1].ShotField();
            List<Coordinate> potentialCoordinates = new List<Coordinate>()
            {
                testData[0,1].Coordinate, testData[1,0].Coordinate, testData[2,1].Coordinate, testData[1,2].Coordinate
            };
            var result = new AI().GetNextShoot(testData);

            Assert.IsTrue(potentialCoordinates.Any(c => c.X == result.X && c.Y == result.Y));
        }

        [TestMethod]
        //testuje dla dwoch trafionych pol
        public void PotentialFields2()
        {
            var testData = Fixtures.GenerateFieldMatrix(5);
            Ship ship = new Ship(new System.Collections.Generic.List<Field>() { testData[1, 1], testData[2, 1], testData[3, 1] });
            testData[1, 1].ShotField();
            testData[2, 1].ShotField();
            List<Coordinate> potentialCoordinates = new List<Coordinate>()
            {
                testData[0,1].Coordinate, testData[3,1].Coordinate
            };
            var result = new AI().GetNextShoot(testData);

            Assert.IsTrue(potentialCoordinates.Any(c => c.X == result.X && c.Y == result.Y));
        }

        [TestMethod]
        //zatopiony statek
        public void PotentialFields3()
        {
            var testData = Fixtures.GenerateFieldMatrix(4);
            Ship ship = new Ship(new System.Collections.Generic.List<Field>() { testData[1, 1], testData[2, 1], testData[3, 1] });
            testData[1, 1].ShotField();
            testData[2, 1].ShotField();
            testData[3, 1].ShotField();
            List<Coordinate> potentialCoordinates = new List<Coordinate>()
            {
                testData[0,3].Coordinate, testData[1,3].Coordinate, testData[2,3].Coordinate, testData[3,3].Coordinate,
            };
            var result = new AI().GetNextShoot(testData);

            Assert.IsTrue(potentialCoordinates.Any(c => c.X == result.X && c.Y == result.Y));
        }

        [TestMethod]
        //testuje dla sytuacji gdy nie został znaleziony żaden statek
        public void PotentialFields4()
        {
            var testData = Fixtures.GenerateFieldMatrix(10);
            Ship ship = new Ship(new System.Collections.Generic.List<Field>() { testData[1, 1], testData[2, 1], testData[3, 1] });

            var result = new AI().GetNextShoot(testData);

            if (result == null) throw new Exception();

            Assert.IsTrue(result != null);
        }
    }
}
