﻿using Statki.Common;
using Statki.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Statki
{
    public class AI
    {
        public Coordinate GetNextShoot(Field[,] fields)
        {
            IEnumerable<Field> allFields = Find(fields, field => true);
            var fieldWithHittedShips = Find(fields, field => field.FieldState == FieldState.Hit && !field.Ship.IsSunk);
            List<Coordinate> potentialCoordinate = new List<Coordinate>();

            if (fieldWithHittedShips.Count() == 1)
            {
                Coordinate singlefield = fieldWithHittedShips.First().Coordinate;
                if (singlefield.X - 1 >= 0)
                {
                    potentialCoordinate.Add(new Coordinate(singlefield.X - 1, singlefield.Y));
                    // lewa
                }
                if (singlefield.X + 1 < fields.GetLength(0))
                {
                    potentialCoordinate.Add(new Coordinate(singlefield.X + 1, singlefield.Y));
                    //prawa
                }
                if (singlefield.Y - 1 >= 0)
                {
                    potentialCoordinate.Add(new Coordinate(singlefield.X, singlefield.Y - 1));
                    // góra planszy
                }
                if (singlefield.Y + 1 < fields.GetLength(1))
                //kiedy dól planszy?
                {
                    potentialCoordinate.Add(new Coordinate(singlefield.X, singlefield.Y + 1));
                    //dól planszy
                }
            }
            else if (fieldWithHittedShips.Count() > 1)
            {
                Direction direction = CalculateDirection(fieldWithHittedShips);
                if (direction == Direction.Horizontal)
                {
                    int y = fieldWithHittedShips.First().Coordinate.Y;
                    int leftX = fieldWithHittedShips.Select(field => field.Coordinate.X).Min();
                    int rightX = fieldWithHittedShips.Select(field => field.Coordinate.X).Max();
                    if (leftX - 1 >= 0)
                    {
                        potentialCoordinate.Add(new Coordinate(leftX - 1, y));
                    }
                    if (rightX + 1 < fields.GetLength(0))
                    {
                        potentialCoordinate.Add(new Coordinate(rightX + 1, y));
                    }
                }
                if (direction == Direction.Vertical)
                {
                    int x = fieldWithHittedShips.First().Coordinate.X;
                    int upY = fieldWithHittedShips.Select(field => field.Coordinate.Y).Min();
                    int downY = fieldWithHittedShips.Select(field => field.Coordinate.Y).Max();
                    if (upY - 1 >= 0)
                    {
                        potentialCoordinate.Add(new Coordinate(x, upY - 1));
                    }
                    if (downY + 1 < fields.GetLength(1))
                    {
                        potentialCoordinate.Add(new Coordinate(x, downY + 1));
                    }
                }
            }
            else
            {
                potentialCoordinate.AddRange(
                    allFields.Where(field => field.FieldState == FieldState.Blank).Select(field => field.Coordinate));
            }

            var occupietFields = allFields.Where(field => field.Ship != null && field.Ship.IsSunk).Select(field => field.Ship).Distinct().SelectMany(ship => ship.GetOccupietFields(fields)).Distinct().ToList();
            potentialCoordinate = potentialCoordinate
                .Where(coord => allFields.Any(field => field.Coordinate.X == coord.X && field.Coordinate.Y == coord.Y && field.FieldState == FieldState.Blank)
                && !occupietFields.Any(field => field.Coordinate.X == coord.X && field.Coordinate.Y == coord.Y)).ToList();

            if (!potentialCoordinate.Any())
            {
                throw new Exception("Brak mozliwych pol do wylosowania.");
            }
            return potentialCoordinate.RandomElement();
        }

        private static IEnumerable<Field> Find(Field[,] fields, Func<Field, bool> func)
        {
            List<Field> founded = new List<Field>();
            foreach (var field in fields)
            {
                if (func(field))
                {
                    founded.Add(field);
                }
            }

            return founded;
        }

        public static Direction CalculateDirection(IEnumerable<Field> fields)
        {
            return CalculateDirection(fields.Select(field => field.Coordinate));
        }

        public static Direction CalculateDirection(IEnumerable<Coordinate> coordinates)
        {
            var x = coordinates.Select(c => c.X);
            return x.Max() == x.Min() ? Direction.Vertical : Direction.Horizontal;
        }
    }
}
